# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['maurice', 'maurice.patchers']

package_data = \
{'': ['*']}

install_requires = \
['dill>=0.3.1,<0.4.0', 'wrapt>=1.12.1,<2.0.0']

extras_require = \
{'all': ['scikit-learn', 'tensorflow>=2.4.0,<3.0.0', 'pandas>=1.0.0,<2.0.0'],
 'pandas': ['pandas>=1.0.0,<2.0.0'],
 'sklearn': ['scikit-learn'],
 'tensorflow': ['tensorflow>=2.4.0,<3.0.0']}

setup_kwargs = {
    'name': 'maurice',
    'version': '0.1.8',
    'description': 'Ship better machine learning projects, faster!',
    'long_description': '# Maurice\n',
    'author': 'Tomas Pereira de Vasconcelos',
    'author_email': 'tomasvasconcelos1@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tpvasconcelos/maurice',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)



import logging
from types import MethodType

import dill
import wrapt

from maurice.caching import CACHE_DIR
from maurice.utils import hash_any

logger = logging.getLogger(__name__)


class CachingMethodWrapper(object):
    def __init__(self, instance: type, method: MethodType, args, kwargs):
        self._instance = instance
        self._method = method
        self._args = args
        self._kwargs = kwargs

        self._path_to_cached_method = CACHE_DIR.joinpath(
            # path to module
            *type(instance).__module__.split("."),
            # class name
            type(instance).__name__,
            # instance state hash
            hash_any(self._get_instance_state()),
            # instance method name
            method.__name__,
            # args and kwargs hash
            hash_any((args, kwargs)),
        )
        self._path_to_state = self._path_to_cached_method.joinpath("state.dill")
        self._path_to_result = self._path_to_cached_method.joinpath("result.dill")

    def _get_instance_state(self):
        if hasattr(self._instance, "__getstate__"):
            state = self._instance.__getstate__()
        else:
            state = self._instance.__dict__
        return state

    def _set_instance_state(self, state) -> None:
        if hasattr(self._instance, "__setstate__"):
            self._instance.__setstate__(state)
        else:
            self._instance.__dict__.update(state)

    def run_wrapped(self):
        if not self._path_to_cached_method.exists():
            result = self._method(*self._args, **self._kwargs)
            logger.info(f"Saving cache to: {self._path_to_cached_method}")
            self._path_to_cached_method.mkdir(parents=True, exist_ok=False)
            self._path_to_state.write_bytes(dill.dumps(self._get_instance_state()))
            self._path_to_result.write_bytes(dill.dumps(result))
            return result
        else:
            logger.info(f"Loading cache from: {self._path_to_cached_method}")
            self._set_instance_state(dill.loads(self._path_to_state.read_bytes()))
            result = dill.loads(self._path_to_result.read_bytes())
        return result


def _caching_method_wrapper(method: MethodType, instance: type, args, kwargs):
    return CachingMethodWrapper(instance=instance, method=method, args=args, kwargs=kwargs).run_wrapped()


def patch_method_with_caching(name: str, cls: type) -> None:
    wrapt.wrap_function_wrapper(module=cls.__module__, name=f"{cls.__name__}.{name}", wrapper=_caching_method_wrapper)
